// Fetch-on-demand conference object.
//
// $Id: conference.cc,v 1.29 1996/08/16 18:00:53 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#include <assert.h>
#include <stdlib.h>
#include <iostream.h>
#include "conference.h"
#include "conf-cache.h"


// ================================================================
// ================================================================
//                            Class Conference


// ================================================================ 
//                    Constructors and Destructors                  


Conference::Conference(Conf_no confno, Conf_cache * cc)
    : cno(confno),
      cache(cc)
{
}

 
Conference::Conference(const Conference &conf)
    : cno(conf.cno),
      cache(conf.cache)
{
}


Conference::~Conference()
{
}


// ================================================================ 
//

const Conference & 
Conference::operator = (const Conference &c)
{
    cno = c.cno;
    cache = c.cache;
    return *this;
}


Status
Conference::prefetch() const
{
    return cache->prefetch(cno);
}

bool
Conference::prefetch_pending() const
{
    return cache->prefetch_pending(cno);
}

Status
Conference::prefetch_name() const
{
    // FIXME: the cache should be able to cache name-only information.
    return cache->prefetch(cno);
}

Kom_err
Conference::errno() const
{
    return cache->get(cno).errno();
}

bool
Conference::present() const
{
    return cache->present(cno);
}

bool
Conference::exists() const
{
    return (bool)(errno() == KOM_NO_ERROR);
}

Conf_no
Conference::conf_no() const
{
    return cno;
}

Connection *
Conference::connection() const
{	
    return cache->connection();
}

Pers_no 
Conference::creator() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).creator();
}

struct tm 
Conference::creation_time() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).creation_time();
}

Text_no 
Conference::presentation() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).presentation();
}

Conf_no 
Conference::supervisor() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).supervisor();
}

Conf_no 
Conference::permitted_submitters() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).permitted_submitters();
}

Conf_no 
Conference::super_conf() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).super_conf();
}

Conf_type 
Conference::type() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).type();
}

struct tm 
Conference::last_written() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).last_written();
}

Text_no 
Conference::msg_of_day() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).msg_of_day();
}

Garb_nice 
Conference::nice() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).nice();
}

LyStr 
Conference::name() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).name();
}

unsigned short 
Conference::no_of_members() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).no_of_members();
}

Local_text_no 
Conference::first_local_no() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).first_local_no();
}

unsigned long 
Conference::no_of_texts() const
{
    assert(errno() == KOM_NO_ERROR);
    return cache->get(cno).no_of_texts();
}


// ================================================================
// ================================================================
//                            Class Conferences


// ================================================================ 
//                    Constructors and Destructors                  


Conferences::Conferences(Connection *co, Async_dispatcher *async_disp)
  : conn(co),
    conf_cache(new Conf_cache(co, async_disp))
{
}
 

// Conferences::Conferences(const Conferences &conf)
//     : conn(conf.conn)
// {
//    // Warning! A refernce counting conf_cache or something similar
//    // is needed if this method is to be implemented!
//     abort();
// }


Conferences::~Conferences()
{
    if (conf_cache)
	delete conf_cache;
}


// ================================================================ 
//

Connection *
Conferences::connection() const
{
    return conn;
}

// const Conferences & 
// Conferences::operator = (const Conferences &)
// {
//     cerr << "Operator= for class Conferences is not yet implemented\n";
//     abort();
// }


Conference
Conferences::operator [](Conf_no confno) const
{
    assert(conf_cache != NULL);
    return Conference(confno, conf_cache);
}

Conf_cache * 
Conferences::read_conf_cache() const
{
    return conf_cache;
}

