// Highlevel handling of a session to a LysKOM server.
//
// $Id: session.h,v 1.42 1996/08/26 10:00:25 ceder Exp $
// Copyright (C) 1994  Per Cederqvist and Inge Wallin
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef KOMXX_SESSION_H
#define KOMXX_SESSION_H

#include <stdlib.h>
#include <iostream.h>

#include "array.h"
#include "LyStr.h"
#include "kom-types.h"
#include "who-info-ident.h"
#include "micro-conf.h"
#include "conf-z-info.h"
#include "connection.h"
#include "async_dispatcher.h"
#include "mark.h"
#include "trinary.h"
#include "error-aggregate.h"
#include "user-area.h"
#include "conference.h"
#include "person.h"
#include "maps.h"
#include "text.h"
#include "membership.h"

class Membership_cache;
class user_active_question;

// +++FIXME: Hur skall felhanteringen g till?  Vi vill kunna f ut
//           en felkod om ngot gr fel.

class Session {
  public:
    Session(Connection *,
	    const LyStr &client_name,
	    const LyStr &client_version);
    ~Session();

    friend ostream &operator <<(ostream &, const Session &);

    Conferences &  conferences();
    Persons     &  persons();
    Maps        &  maps();
    Texts       &  texts();
    Pers_no        my_login() const;
    Session_no     my_session() const;

    // Create a new person.  If RESULT is non-NULL, the new person number will
    // be stored there.  An automatic login is performed if you are not already
    // logged in.  If AUTOLOGIN is non-NULL, it will be set if an auto-login
    // was performed.
    Status              create_person(const LyStr &name, const LyStr &password,
				      Pers_no *result = NULL, 
				      bool *autologin = NULL);

    Status              login(Pers_no, const LyStr &, bool invisible = false);
    Status              logout();
    Status              add_member(Conf_no cno, Pers_no reader,
				   unsigned char priority, 
				   unsigned short placement);
    Kom_err		sub_member(Conf_no cno, Pers_no member);
    Status              send_message(Pers_no recipient, const LyStr &message);
    struct tm           get_time(); 
    Status              change_what_i_am_doing(const LyStr &str);
    Array<Who_info_ident> who_is_on();
    Server_info         get_server_info();
    Status              set_motd_of_lyskom(Text_no motd);
    Kom_err		set_presentation(Conf_no cno, Text_no tno);

    Kom_res<Text_no>	get_last_text(const struct tm &);
    bool		is_supervisor(Pers_no supervisor, Conf_no conf);

    // Membership handling.

    // Is person a member of conf?
    bool                is_member(Pers_no person, Conf_no conf);

    // Prefetch information so that is_member can be answered.
    Status              prefetch_is_member(Pers_no person, Conf_no conf);

    Membership          get_membership(Pers_no person, Conf_no conf);
    Status              prefetch_membership(Pers_no person, Conf_no conf);
    bool                prefetch_membership_pending(Pers_no, Conf_no);
    bool                membership_present(Pers_no, Conf_no);
    Kom_err		set_last_read(Conf_no cno,
				      Local_text_no last_text_read);

    // Tell the server that the user was active.  You can call this
    // method as often as you like -- it will only send a user-active
    // question if at least 30 seconds have passed since the previous
    // user-active sent, and the reply to the previous user-active has
    // been received.  This method never blocks, except possibly the
    // first time it is called.
    void		user_active();
    Array<Conf_z_info>  lookup_z_name(const LyStr &pattern,
				      bool want_persons =true,
				      bool want_confs =true);
    Array<Mark>         get_marks();
    Kom_res<Local_text_no> last_local_before(Conf_no, const struct tm &);

    // Get a user-area handler.  The handler class is owned by the
    // session, and deleted by it when the session is deleted or when
    // the user is logged out.  In other words: don't hold on to the
    // pointer returned by user_area too long!
    Kom_res<User_area*> get_user_area();
    
    // Obsolete:
    Array<Micro_conf>   lookup_name(const LyStr &pattern);
    Array<Pers_no>      regexp_lookup_persons(const LyStr &pattern);

    Async_dispatcher  & async_dispatcher();
    Status              handle_async(int this_many = 1);

    Connection         *connection() const;
  private:
    Connection  * conn;
    Pers_no       logged_in_person;
    Session_no    session_number; // 0 if not yet known

    Conferences * all_confs;
    Persons     * all_persons;
    Maps        * all_maps;
    Texts       * all_texts;
    User_area   * my_user_area;
    Membership_cache *all_memberships;

    Async_dispatcher  * m_async_dispatcher;

    trinary server_supports_set_last_read;
    trinary server_supports_user_active;

    time_t previous_user_active_time;
    user_active_question *pending_user_active;

    // From time to time, this class issues questions whose results it
    // doesn't care about.  They are pushed on this array, and once in
    // a while the session uses check_ignored() to remove those
    // questions whose result has returned.
    Array<question*> ignored_questions;
    void check_ignored();

    // Not implemented, since it doesn't make sense to copy sessions.
    Session(const Session &);
    const Session & operator = (const Session &);
};


#endif
