# $Id: toolbar.py,v 1.12 1996/08/21 10:01:04 ceder Exp $
# Copyright (C) 1996  Per Cederqvist and Inge Wallin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

import komhtml
import string
from language import lingo

system_error='system_error'

class toolbar:
    """Toolbar: the line of icons at the top (and bottom) of each page.

    The toolbar has this look, from left to right:

    default: typically "read next text"
    std:    go home, go conf, etc
    mode-special: e.g. write comment
    kill:    restart with force"""

    def __init__(self):
	self.top_wanted = 1
	self.bottom_wanted = 0
	self.default = []
	self.std = []
	self.toolbar = []

    def fix_std(self):
	if not self.std:
	    self.std = [komhtml.icon("home", lingo.alt_home(),
				     komhtml.komurl("home"),
				     lingo.explain_home()),
			komhtml.icon("selconf", lingo.alt_select_conf(),
				     komhtml.komurl("select_conf_form"),
				     lingo.explain_select_conf())]
	    self.kill = [komhtml.icon("exit", lingo.alt_kill(),
				      komhtml.komurl("kill"),
				      lingo.explain_kill())]

    def enable_bottom(self):
	self.bottom_wanted = 1

    def disable_top(self):
	self.top_wanted = 0

    def add_tool(self, image, alt, url, explanation):
	self.toolbar.append(komhtml.icon(image, alt, url, explanation))

    def add_text_unformatted(self, tno):
	self.add_tool("unformatted", lingo.alt_unformatted(),
		      komhtml.komurl("text?text_no=" + `tno` + "&style=pre"),
		      lingo.explain_unformatted(tno))

    def add_text_formatted(self, tno):
	self.add_tool("formatted", lingo.alt_formatted(),
		      komhtml.komurl("text?text_no=" + `tno`
				     + "&style=heuristic"),
		      lingo.explain_formatted(tno))

    def add_config(self):
	self.add_tool("config", lingo.alt_config(),
		      komhtml.komurl("configuration"),
		      lingo.explain_config())

    def add_dflt(self, image, alt, url, explanation):
	if len(self.default) != 0:
	    raise system_error, "default set twice"
	self.default = [komhtml.icon(image, alt, url, explanation)]

    def add_comment(self, textno):
	self.add_tool("comment", lingo.alt_comment(),
		      komhtml.komurl("comment?parent=" + `textno`),
		      lingo.explain_comment(textno))

    def add_current_conf(self, confno):
	self.add_tool("board", lingo.alt_conf(),
		      komhtml.komurl("conf?conf_no=" + `confno`),
		      lingo.explain_conf())

    def add_write(self, confno):
	self.add_tool("write", lingo.alt_write_text(),
		      komhtml.komurl("write_text?conf_no=" + `confno`),
		      lingo.explain_write_text())


    def add_default_conf(self, cno, cname):
	self.add_dflt("next-conf", lingo.alt_next_conf(cname),
		      komhtml.komurl("conf?conf_no=" + `cno`),
		      lingo.explain_next_conf(cname))

    def add_default_text(self, tno):
	self.add_dflt("next-text", lingo.alt_next_text(tno),
		      komhtml.komurl("text?text_no=" + `tno`),
		      lingo.explain_next_text(tno))

    def add_default_comment(self, tno):
	self.add_dflt("next-comment", lingo.alt_next_comment(tno),
		      komhtml.komurl("text?text_no=" + `tno`),
		      lingo.explain_next_comment(tno))

    def add_default_footnote(self, tno):
	self.add_dflt("next-footnote", lingo.alt_next_footnote(tno),
		      komhtml.komurl("text?text_no=" + `tno`),
		      lingo.explain_next_footnote(tno))

    def add_default_time(self):
	self.add_dflt("time", lingo.alt_see_time(),
		      komhtml.komurl("time"),
		      lingo.explain_see_time())

    def at_top(self):
	if self.top_wanted:
	    return self.as_string()
	else:
	    return ""

    def as_string(self):
	self.fix_std()
	return "<p>" + string.join(self.default + self.std
				   + self.toolbar + self.kill, "\n") + "\n<p>"


    def at_bottom(self):
	if self.bottom_wanted:
	    return self.as_string()
	else:
	    return ""
